SUMMARY_DOCS = """
api：每日60秒早报
api描述：获取每日的60秒早报接口,每天凌晨一点更新

返回参数:
名称   描述
news	早报新闻列表
weiyu	微语
image	早报图片
head_image	早报头图
date	早报日期

你收到上述api返回请求：
{morning_news}

如果api访问成功，你返回的内容必须包括4部分内容：
首先告诉我早报日期和微语，然后总结早报新闻列表：你需要罗列当天有多少条早报新闻，并且高质量地总结每条新闻的细节
最后把早报图片的url发送给我

返回的内容应尽可能方便可读，你不能编造信息
"""
