import logging

LOGGING_LEVEL = logging.INFO

LOGGING_FMT = '[%(levelname)s][%(asctime)s][%(filename)s:%(lineno)d] - %(message)s'
LOGGING_DATEFMT = '%Y-%m-%d %H:%M:%S'

ALL_MAX_TOKENS_NUM = 4096

MEMORY_MAX_TOKENS_NUM = 1000
BOT_SCRATCHPAD_MAX_TOKENS_NUM = 2000
